import { Component, OnInit } from '@angular/core';
import { MovesService } from '../moves.service';
import { Card } from '../card';
import { CardComponent } from '../card/card.component';

@Component({
  selector: 'app-board',
  standalone: true,
  imports: [CardComponent],
  templateUrl: './board.component.html',
  styleUrl: './board.component.css',
})
export class BoardComponent implements OnInit {
  cards!: Card[]; // sisältää korttioliot, joita käytetään pelilaudalla
  firstOpenedCard!: number; // näyttää uuden kuvaparin etsinnän ensimmäiseksi avatun kuvan indeksin
  secondOpenedCard!: number; // näyttää uuden kuvaparin etsinnän toiseksi avatun kuvan indeksin
  movesCount!: number; // tehtyjen siirtojen määrä
  haltMoves!: boolean; // jos arvo true, siirtoja ei voida tehdä
  cardsExists!: boolean; // jos arvo false, peli päättyy
  showWinText!: boolean; // jos arvo true, näytetään voittoteksti

  constructor(private movesService: MovesService) {}

  ngOnInit(): void {
    this.newGame();
  }
  // alustetaan muuttujiin pelin alkutilanne
  newGame() {
    this.showWinText = false;
    this.cardsExists = true;
    this.haltMoves = false;
    this.movesCount = 0;
    this.movesService.initMoves(); // alustetaan tehtyjen siirtojen määrä
    this.firstOpenedCard = 99; // alustus indeksiin jota ei ole
    this.secondOpenedCard = 99;
    this.cards = [
      { value: 'α', status: 'hidden' },
      { value: 'α', status: 'hidden' },
      { value: 'β', status: 'hidden' },
      { value: 'β', status: 'hidden' },
      { value: 'γ', status: 'hidden' },
      { value: 'γ', status: 'hidden' },
      { value: 'δ', status: 'hidden' },
      { value: 'δ', status: 'hidden' },
      { value: 'ε', status: 'hidden' },
      { value: 'ε', status: 'hidden' },
      { value: 'λ', status: 'hidden' },
      { value: 'λ', status: 'hidden' },
      { value: 'μ', status: 'hidden' },
      { value: 'μ', status: 'hidden' },
      { value: 'π', status: 'hidden' },
      { value: 'π', status: 'hidden' },
    ];
    this.shuffleArray(this.cards);
  }
  // sekoitetaan taulukossa olevat alkiot satunnaiseen
  // järjestykseen
  shuffleArray(array: any[]) {
    array.sort(() => Math.random() - 0.5);
    console.log(array);
  }

  // Siirto, eli kuvaparin avaaminen
  makeMove(index: number): boolean | null {
    // Jos klikattu kuva on näkyvillä tai poistettu, tai haltMoves arvo on true,
    // siirtoa ei voida tehdä eli poistutaan metodista return false:lla
    if (
      this.cards[index].status === 'visible' ||
      this.cards[index].status === 'removed' ||
      this.haltMoves
    ) {
      return false; // poistutaan
    }
    // Kun cardia klikataan, asetetaan status näkyväksi
    this.cards[index].status = 'visible';

    // Kuvaparin etsintä
    // tarkistetaan onko kyseessä 1. vai 2. kortin avaus:
    /* 1. avaus, sillä jos firstOpenedCard on 99,
       niin sitä ei ole vielä avattu. */
    if (this.firstOpenedCard === 99) {
      // sijoitetaan avatun kortin indeksi muuttujaan firstOpenedCard
      this.firstOpenedCard = index;
      //2. avaus
    } else {
      this.movesService.addMove(); // lisätään siirto tehtyjen siirtojen määrään
      this.movesCount = this.movesService.getMoves(); // päivitetään tehtyjen siirtojen määrä
      this.secondOpenedCard = index;

      this.checkMatch();
    }
    return true;
  }

  // Tarkistetaan onko avatut kortit samanlaisia
  checkMatch() {
    // jos ovat erilaisia, piilotetaan kuvat
    if (
      this.cards[this.firstOpenedCard].value !==
      this.cards[this.secondOpenedCard].value
    ) {
      /* Tehdään viive-efekti, eli kuvat näkyvät yhden sekunnin.
         viiveen ajaksi asetetaan haltMoves-arvoksi true, jotta uusia
         siirtoja ei voida tehdä */
      this.haltMoves = true;
      // viiveen mentyä ohi, kortit piilotetaan ja alustetaan, ja
      // siirtoja voi taas tehdä
      setTimeout(() => {
        this.cards[this.firstOpenedCard].status = 'hidden';
        this.cards[this.secondOpenedCard].status = 'hidden';
        this.firstOpenedCard = 99;
        this.secondOpenedCard = 99;
        this.haltMoves = false;
      }, 1000); // viive 1000ms, eli 1 sekunti
    } else {
      /* jos avatun kuvaparin kuvat ovat samanlaisia, ne poistetaan 
         pelilaudalta, eli asetetaan statukseksi 'removed' */
      this.haltMoves = true;
      // viiveen ajan kortit näkyvät, mutta uusia siirtoja ei voi tehdä
      setTimeout(() => {
        this.cards[this.firstOpenedCard].status = 'removed';
        this.cards[this.secondOpenedCard].status = 'removed';
        this.firstOpenedCard = 99;
        this.secondOpenedCard = 99;
        this.checkWin(); // tarkistetaan onko pelaaja päässyt pelin läpi
        this.haltMoves = false;
      }, 1000);
    }
  }
  // Tarkistetetaan onko peli voitettu eli pelattu loppuun
  checkWin(): boolean {
    // Aluksi oletamme että pelilaudalla ei ole kortteja
    this.cardsExists = false;
    /* käydään läpi cards-taulukon kortit, ja jos löytyy kortti
       joka ei ole 'removed', on kortteja jäljellä, eli 
       peli ei ole päättynyt */
    this.cards.forEach((c) => {
      if (c.status !== 'removed') {
        // kortteja on jäljellä
        this.cardsExists = true;
      }
    });
    if (this.cardsExists) {
      return false; // poistutaan, peli on kesken
    } else {
      // peli on päättynyt ja näytetään voittoteksti
      this.showWinText = true;
      return true;
    }
  }
}
